"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class PreviewCommand {
    constructor($bundleValidatorHelper, $liveSyncService, $networkConnectivityValidator, $projectData, $options, $playgroundQrCodeGenerator, $previewCommandHelper) {
        this.$bundleValidatorHelper = $bundleValidatorHelper;
        this.$liveSyncService = $liveSyncService;
        this.$networkConnectivityValidator = $networkConnectivityValidator;
        this.$projectData = $projectData;
        this.$options = $options;
        this.$playgroundQrCodeGenerator = $playgroundQrCodeGenerator;
        this.$previewCommandHelper = $previewCommandHelper;
        this.allowedParameters = [];
    }
    execute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            this.$previewCommandHelper.run();
            yield this.$liveSyncService.liveSync([], {
                syncToPreviewApp: true,
                projectDir: this.$projectData.projectDir,
                skipWatcher: !this.$options.watch,
                watchAllFiles: this.$options.syncAllFiles,
                clean: this.$options.clean,
                bundle: !!this.$options.bundle,
                release: this.$options.release,
                env: this.$options.env,
                timeout: this.$options.timeout,
                useHotModuleReload: this.$options.hmr
            });
            yield this.$playgroundQrCodeGenerator.generateQrCodeForCurrentApp({ useHotModuleReload: this.$options.hmr });
        });
    }
    canExecute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.$networkConnectivityValidator.validate();
            this.$bundleValidatorHelper.validate();
            return true;
        });
    }
}
exports.PreviewCommand = PreviewCommand;
$injector.registerCommand("preview", PreviewCommand);
